package work

import (
	"fmt"
	"strings"
	"sync"
	"sync/atomic"
)

type FloatMetric struct {
	mux  sync.RWMutex
	name string
	val  atomic.Value
}

func NewFlotMetric(name string) *FloatMetric {
	m := &FloatMetric{name: name}
	m.val.Store(float64(0.0))
	return m
}

func (m *FloatMetric) Incr(n float64) {
	m.mux.Lock()
	defer m.mux.Unlock()
	v := m.val.Load().(float64)
	m.val.Store(v + n)
}

func (m *FloatMetric) Get() float64 {
	return m.val.Load().(float64)
}

type Metric struct {
	mux  sync.RWMutex
	name string
	val  atomic.Value
}

func NewMetric(name string) *Metric {
	m := &Metric{name: name}
	m.val.Store(0)
	return m
}

func (m *Metric) Incr() {
	m.mux.Lock()
	defer m.mux.Unlock()
	v := m.val.Load().(int)
	m.val.Store(v + 1)
}

func (m *Metric) Get() int {
	return m.val.Load().(int)
}

func (m *Metric) Set(v int) {
	m.mux.Lock()
	defer m.mux.Unlock()
	m.val.Store(v)
}

func (m *Metric) Add(v int) {
	m.mux.Lock()
	defer m.mux.Unlock()
	current := m.val.Load().(int)
	m.val.Store(current + v)
}

var (
	totalUser    = NewMetric("totalUser")
	failedUser   = NewMetric("failedUser")
	finishedUser = NewMetric("finishedUser")
	totalTx      = NewMetric("totalTx")
	failedTx     = NewMetric("failedTx")
	finishedTx   = NewMetric("finishedTx")

	costToken    = NewFlotMetric("costToken")
	receiveToken = NewFlotMetric("receiveToken")
)

func SetTotalUser(cnt int) {
	totalUser.Set(cnt)
}

func IncrFinishedUser() {
	finishedUser.Incr()
}

func IncrFailedUser() {
	failedUser.Incr()
}

func IncrTotalTx() {
	totalTx.Incr()
}

func IncrFailedTx() {
	failedTx.Incr()
}

func IncrFinishedTx() {
	finishedTx.Incr()
}

func IncrReceiveToken(val float64) {
	receiveToken.Incr(val)
}

func IncrCostToken(val float64) {
	costToken.Incr(val)
}

func GetReport() string {
	str := make([]string, 0)
	str = append(str, "REPORT\t\t\tFailed/Total\t\t\tFinished/Total")
	str = append(str, fmt.Sprintf("USER\t\t\t%5d/%-5d\t\t\t\t%5d/%-5d", failedUser.Get(), totalUser.Get(), finishedUser.Get(), totalUser.Get()))
	str = append(str, fmt.Sprintf("TX\t\t\t\t\t\t\t\t%5d/%-5d", finishedTx.Get(), totalTx.Get()))
	str = append(str, fmt.Sprintf("%f Cost / %f Receive", costToken.Get(), receiveToken.Get()))
	return strings.Join(str, "\n")
}
